@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
+/* Copyright (c) The Exim Maintainers 2020 - 2023 */
 /* Copyright (c) Jeremy Harris 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -15,25 +15,21 @@ included in the main Exim build */
 
 unsigned mp_index = 0;
 
-/* Global dummy variables and functions */
+/* Global dummy variables */
 
 void fn_smtp_receive_timeout(const uschar * name, const uschar * str) {}
 uschar * syslog_facility_str;
 
-/* Solaris needs this one for the macro expand_string() */
-const uschar * expand_string_2(const uschar * string, BOOL * textonly_p)
-{return NULL; }
-
 /******************************************************************************/
 
 void
 builtin_macro_create_var(const uschar * name, const uschar * val)
 {
-printf ("static macro_item p%u = { ", mp_index);
+printf ("static macro_item p%d = { ", mp_index);
 if (mp_index == 0)
   printf(".next=NULL,");
 else
-  printf(".next=&p%u,", mp_index-1);
+  printf(".next=&p%d,", mp_index-1);
 
 printf(" .command_line=FALSE, .namelen=%d, .replen=%d,"
 	" .name=US\"%s\", .replacement=US\"%s\" };\n",
@@ -135,6 +131,9 @@ due to conflicts with other common macros. */
 #ifdef EXPAND_DLFUNC
   builtin_macro_create(US"_HAVE_DLFUNC");
 #endif
+#ifdef USE_TCP_WRAPPERS
+  builtin_macro_create(US"_HAVE_TCPWRAPPERS");
+#endif
 #ifndef DISABLE_TLS
   builtin_macro_create(US"_HAVE_TLS");
 # ifdef USE_GNUTLS
@@ -161,9 +160,6 @@ due to conflicts with other common macros. */
 #ifndef DISABLE_DNSSEC
   builtin_macro_create(US"_HAVE_DNSSEC");
 #endif
-#ifndef DISABLE_ESMTP_LIMITS
-  builtin_macro_create(US"_HAVE_ESMTP_LIMITS");
-#endif
 #ifndef DISABLE_EVENT
   builtin_macro_create(US"_HAVE_EVENT");
 #endif
@@ -191,11 +187,8 @@ due to conflicts with other common macros. */
 #ifdef TCP_FASTOPEN
   builtin_macro_create(US"_HAVE_TCP_FASTOPEN");
 #endif
-#ifdef EXIM_HAVE_SPF
+#ifdef SUPPORT_SPF
   builtin_macro_create(US"_HAVE_SPF");
-# ifdef EXPERIMENTAL_SPF_PERL
-  builtin_macro_create(US"_HAVE_EXPERIMENTAL_SPF_PERL");
-# endif
 #endif
 #ifdef SUPPORT_SRS
   builtin_macro_create(US"_HAVE_SRS");
@@ -218,25 +211,10 @@ due to conflicts with other common macros. */
 #ifndef DISABLE_TLS_RESUME
   builtin_macro_create(US"_HAVE_TLS_RESUME");
 #endif
-#ifndef DISABLE_WELLKNOWN
-  builtin_macro_create(US"_HAVE_WELLKNOWN");
-#endif
 #ifdef EXPERIMENTAL_XCLIENT
   builtin_macro_create(US"_HAVE_XCLIENT");
 #endif
 
-#ifdef USE_SQLITE
-  builtin_macro_create(US"_HAVE_HINTS_SQLITE");
-#elif defined(USE_TDB)
-  builtin_macro_create(US"_HAVE_HINTS_TDB");
-#elif defined(USE_DB)
-  builtin_macro_create(US"_HAVE_HINTS_BDB");
-#elif defined(USE_GDBM)
-  builtin_macro_create(US"_HAVE_HINTS_GDBM");
-#else
-  builtin_macro_create(US"_HAVE_HINTS_NDBM");
-#endif
-
 #ifdef LOOKUP_LSEARCH
   builtin_macro_create(US"_HAVE_LOOKUP_LSEARCH");
 #endif
@@ -331,6 +309,9 @@ exp_features(void)
 #ifdef EXPERIMENTAL_DSN_INFO
   builtin_macro_create(US"_EXP_DSNI");
 #endif
+#ifdef EXPERIMENTAL_ESMTP_LIMITS
+  builtin_macro_create(US"_EXP_LIMITS");
+#endif
 #ifdef EXPERIMENTAL_QUEUEFILE
   builtin_macro_create(US"_EXP_QUEUEFILE");
 #endif
@@ -369,7 +350,7 @@ options();
 expansions();
 params();
 
-printf("macro_item * macros = &p%u;\n", mp_index-1);
+printf("macro_item * macros = &p%d;\n", mp_index-1);
 printf("macro_item * mlast = &p0;\n");
 exit(0);
 }
