@@ -2,7 +2,6 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2024 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -32,15 +31,15 @@ uschar *
 rf_expand_data(address_item *addr, uschar *s, int *prc)
 {
 uschar *yield = expand_string(s);
-if (yield) return yield;
+if (yield != NULL) return yield;
 if (f.expand_string_forcedfail)
   {
-  DEBUG(D_route) debug_printf("forced failure for expansion of %q\n", s);
+  DEBUG(D_route) debug_printf("forced failure for expansion of \"%s\"\n", s);
   *prc = DECLINE;
   }
 else
   {
-  addr->message = string_sprintf("failed to expand %q: %s", s,
+  addr->message = string_sprintf("failed to expand \"%s\": %s", s,
     expand_string_message);
   *prc = DEFER;
   }
