@@ -2,7 +2,6 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2025 */
 /* Copyright (c) Andrew Colin Kissa <andrew@topdog.za.net> 2016 */
 /* Copyright (c) University of Cambridge 2016 */
 /* See the file NOTICE for conditions of use and distribution. */
@@ -25,7 +24,7 @@ extern queuefile_transport_options_block queuefile_transport_option_defaults;
 
 /* The main and init entry points for the transport */
 
-extern void queuefile_transport_init(driver_instance *);
 extern BOOL queuefile_transport_entry(transport_instance *, address_item *);
+extern void queuefile_transport_init(transport_instance *);
 
 /* End of transports/queuefile.h */
